Subject: fix for vulnerability CVE-2011-3341 for OpenTTD 0.6.0 - 0.6.3 (Denial of service via improperly validated commands)
From: OpenTTD developer team <info@openttd.org>
Origin: backport, https://github.com/OpenTTD/OpenTTD/commit/4836a6e
Bug: https://github.com/OpenTTD/OpenTTD/issues/4745

In multiple places in-game commands are not properly validated that allow remote 
attackers to cause a denial of service (crash) and possibly execute arbitrary 
code via unspecified vectors.

The bug is exploitable only in-game so the attacker must have access to the 
server: his IP must not be banned, he must know the password if it has been set 
and the server must not be full.

The major cause of these bugs are off-by-one errors in the validation of the 
sent commands from the clients to the server, and from the server to the client. 
One could therefore, in theory, affect both the server and the clients of that 
server.

Two of the cases (since 0.7.0) are known to make the game state invalid, which 
causes an eventual crash of the application via an “abort()”. Two cases cause a 
read beyond the boundaries of a (static) table (resp. since 0.3.5 and 1.0.0). 
The last case allows changes to the game state of others that might trigger 
invalid reads for other players if they had the autoreplace window opened (since 
0.6.0).


Index: src/network/network_data.cpp
===================================================================
--- src/network/network_data.cpp	(revision 22703)
+++ src/network/network_data.cpp	(working copy)
@@ -96,7 +96,7 @@
 	_current_player = cp->player;
 	_cmd_text = cp->text;
 	/* cp->callback is unsigned. so we don't need to do lower bounds checking. */
-	if (cp->callback > _callback_table_count) {
+	if (cp->callback >= _callback_table_count) {
 		DEBUG(net, 0, "Received out-of-bounds callback (%d)", cp->callback);
 		cp->callback = 0;
 	}
Index: src/players.cpp
===================================================================
--- src/players.cpp	(revision 22703)
+++ src/players.cpp	(working copy)
@@ -703,7 +703,7 @@
 			GroupID id_g = GB(p1, 16, 16);
 			CommandCost cost;
 
-			if (!IsValidGroupID(id_g) && !IsAllGroupID(id_g) && !IsDefaultGroupID(id_g)) return CMD_ERROR;
+			if (IsValidGroupID(id_g) ? GetGroup(id_g)->owner != _current_company : !IsAllGroupID(id_g) && !IsDefaultGroupID(id_g)) return CMD_ERROR;
 			if (new_engine_type != INVALID_ENGINE) {
 				/* First we make sure that it's a valid type the user requested
 				 * check that it's an engine that is in the engine array */

